#pragma comment(linker, "/version:1.0") // MUST be in the form of major.minor

#include <windows.h>
#include <shlwapi.h>
#include <commctrl.h>

#define countof(x) (sizeof(x)/sizeof(x[0]))

typedef struct {
	UINT cchUsed;
	TCHAR sz[0x10000];
} OUTBUF, *POUTBUF;

VOID WINAPI PrintBuffer( POUTBUF pBuf, PCTSTR szFormat, ... );

UINT __fastcall EntryMain( UINT argc, PTSTR *argv )
{
	// Both "/silent" and "-silent" should be valid.
	BOOLEAN fSilent = ( argc == 2 && (argv[1][0] | 0x02) == TEXT('/') &&
	                    StrCmpI(argv[1] + 1, TEXT("silent")) == 0 );

	HKEY hKeyMountPoints;

	POUTBUF pBuf = LocalAlloc(LMEM_ZEROINIT, sizeof(OUTBUF));
	if (!pBuf) return(~0);

	if (RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\MountPoints2"),
	                 0, KEY_ENUMERATE_SUB_KEYS, &hKeyMountPoints) == ERROR_SUCCESS)
	{
		static const PCTSTR pszValues[] =
		{
			TEXT("_CommentFromDesktopINI"),
			TEXT("_LabelFromDesktopINI"),
		};

		TCHAR szKeyName[256]; // key names are limited to 255 characters
		DWORD cchKeyName = countof(szKeyName);
		DWORD dwIndex = 0;

		while (RegEnumKeyEx(hKeyMountPoints, dwIndex, szKeyName, &cchKeyName, NULL, NULL, NULL, NULL) == ERROR_SUCCESS)
		{
			HKEY hKeyMount;

			if (StrCmpN(szKeyName, TEXT("##"), 2) == 0 &&
			    RegOpenKeyEx(hKeyMountPoints, szKeyName, 0, KEY_QUERY_VALUE | KEY_SET_VALUE, &hKeyMount) == ERROR_SUCCESS)
			{
				UINT i;

				PrintBuffer(pBuf, TEXT("%s\n"), szKeyName);

				for (i = 0; i < countof(pszValues); ++i)
				{
					LONG lResult = RegQueryValueEx(hKeyMount, pszValues[i], NULL, NULL, NULL, NULL);

					if (lResult == ERROR_FILE_NOT_FOUND)
					{
						if (RegSetValueEx(hKeyMount, pszValues[i], 0, REG_SZ, (PBYTE)TEXT(""), sizeof(TCHAR)) == ERROR_SUCCESS)
							PrintBuffer(pBuf, TEXT("\t%s successfully set\n"), pszValues[i]);
						else
							PrintBuffer(pBuf, TEXT("\t%s not found, but could not be set\n"), pszValues[i]);
					}
					else if (lResult == ERROR_SUCCESS)
						PrintBuffer(pBuf, TEXT("\t%s found; no action taken\n"), pszValues[i]);
					else
						PrintBuffer(pBuf, TEXT("\t%s inaccessible; no action taken\n"), pszValues[i]);
				}

				PrintBuffer(pBuf, TEXT("\n"));

				RegCloseKey(hKeyMount);
			}

			cchKeyName = countof(szKeyName);
			++dwIndex;
		}

		RegCloseKey(hKeyMountPoints);
	}

	if (!fSilent)
	{
		InitCommonControls();

		if (!pBuf->cchUsed)
			PrintBuffer(pBuf, TEXT("No records of mapped drives were found."));
		else
			pBuf->sz[pBuf->cchUsed - 2] = 0; // trim away the two trailing endlines

		MessageBox(NULL, pBuf->sz, TEXT("Results"), MB_OK | MB_SETFOREGROUND);
	}

	LocalFree(pBuf);

	return(0);
}

VOID WINAPI PrintBuffer( POUTBUF pBuf, PCTSTR szFormat, ... )
{
	INT cchWritten;

	va_list args;
	va_start(args, szFormat);

	cchWritten = wvnsprintf(
		pBuf->sz + pBuf->cchUsed,
		countof(pBuf->sz) - pBuf->cchUsed - 1,
		szFormat,
		args
	);

	if (cchWritten > 0)
	{
		pBuf->cchUsed += cchWritten;
		pBuf->sz[pBuf->cchUsed] = 0; // provide the NULL-termination that wvnsprintf couldn't guarantee
	}
}
